@@ -29,13 +29,19 @@ unset PNETCDF_HINTS
 
 for i in ${check_PROGRAMS} ; do
     for j in ${safe_modes} ; do
+    for intra_aggr in 0 1 ; do
         if test "$j" = 1 ; then # test only in safe mode
            export PNETCDF_HINTS="romio_no_indep_rw=true"
+        else
+           export PNETCDF_HINTS=
+        fi
+        if test "$intra_aggr" = 1 ; then
+           export PNETCDF_HINTS="${PNETCDF_HINTS};nc_num_aggrs_per_node=2"
         fi
         export PNETCDF_SAFE_MODE=$j
         # echo "set PNETCDF_SAFE_MODE ${PNETCDF_SAFE_MODE}"
 
-        ${MPIRUN} ./$i -q ${TESTOUTDIR}/$i.
+        ${MPIRUN} ./$i -q -f ${TESTOUTDIR}/$i.
 
         if test $? = 0 ; then
            echo "PASS: F90 parallel run on $1 processes --------------- $i"
@@ -55,7 +61,7 @@ for i in ${check_PROGRAMS} ; do
            # echo "test burst buffering feature"
            saved_PNETCDF_HINTS=${PNETCDF_HINTS}
            export PNETCDF_HINTS="${PNETCDF_HINTS};nc_burst_buf=enable;nc_burst_buf_dirname=${TESTOUTDIR};nc_burst_buf_overwrite=enable"
-           ${MPIRUN} ./$i -q ${TESTOUTDIR}/$i.bb.
+           ${MPIRUN} ./$i -q -f ${TESTOUTDIR}/$i.bb.
            if test $? = 0 ; then
               echo "PASS: F90 parallel run on $1 processes --------------- $i"
            fi
@@ -72,6 +78,7 @@ for i in ${check_PROGRAMS} ; do
            ${MPIRUN} ${NCMPIDIFF} -q ${TESTOUTDIR}/$i.ncmpi_plt_crn_0000.nc ${TESTOUTDIR}/$i.bb.ncmpi_plt_crn_0000.nc
         fi
     done
+    done
     rm -f ${OUTDIR}/$i.ncmpi_chk_0000.nc
     rm -f ${OUTDIR}/$i.ncmpi_plt_cnt_0000.nc
     rm -f ${OUTDIR}/$i.ncmpi_plt_crn_0000.nc
