@@ -67,19 +67,64 @@
 #define NX 10
 #define NDIMS 2
 
-#define ERR {if(err!=NC_NOERR){printf("Error at line %d in %s: %s\n", __LINE__,__FILE__, ncmpi_strerror(err));nerrs++;}}
+static int verbose;
+
+#define ERR {if(err!=NC_NOERR){printf("Error at %s:%d : %s\n", __FILE__,__LINE__, ncmpi_strerror(err));nerrs++;}}
 
 #define ERRS(n,a) { \
     int _i; \
     for (_i=0; _i<(n); _i++) { \
         if ((a)[_i] != NC_NOERR) { \
-            printf("Error at line %d in %s: err[%d] %s\n", __LINE__,__FILE__, _i, \
+            printf("Error at %s:%d : err[%d] %s\n", __FILE__,__LINE__, _i, \
                    ncmpi_strerror((a)[_i])); \
                    nerrs++; \
         } \
     } \
 }
 
+static void
+usage(char *argv0)
+{
+    char *help =
+    "Usage: %s [-h] | [-q] [file_name]\n"
+    "       [-h] Print help\n"
+    "       [-q] Quiet mode (reports when fail)\n"
+    "       [filename] output netCDF file name\n";
+    fprintf(stderr, help, argv0);
+}
+
+/*----< pnetcdf_check_mem_usage() >------------------------------------------*/
+/* check PnetCDF library internal memory usage */
+static int
+pnetcdf_check_mem_usage(MPI_Comm comm)
+{
+    int err, nerrs=0, rank;
+    MPI_Offset malloc_size, sum_size;
+
+    MPI_Comm_rank(comm, &rank);
+
+    /* print info about PnetCDF internal malloc usage */
+    err = ncmpi_inq_malloc_max_size(&malloc_size);
+    if (err == NC_NOERR) {
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && verbose)
+            printf("maximum heap memory allocated by PnetCDF internally is %lld bytes\n",
+                   sum_size);
+
+        /* check if there is any PnetCDF internal malloc residue */
+        err = ncmpi_inq_malloc_size(&malloc_size);
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && sum_size > 0)
+            printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
+                   sum_size);
+    }
+    else if (err != NC_ENOTENABLED) {
+        printf("Error at %s:%d: %s\n", __FILE__,__LINE__,ncmpi_strerror(err));
+        nerrs++;
+    }
+    return nerrs;
+}
+
 static MPI_Offset *** calloc_3D(size_t z, size_t y, size_t x)
 {
     if (z*y*x == 0) return NULL;
@@ -102,17 +147,6 @@ static void free_3D(MPI_Offset ***buf)
     free(buf);
 }
 
-static void
-usage(char *argv0)
-{
-    char *help =
-    "Usage: %s [-h] | [-q] [file_name]\n"
-    "       [-h] Print help\n"
-    "       [-q] Quiet mode (reports when fail)\n"
-    "       [filename] output netCDF file name\n";
-    fprintf(stderr, help, argv0);
-}
-
 static int check_contents(int ncid, int *varid)
 {
     int i, j, err, nerrs=0;
@@ -156,7 +190,7 @@ int main(int argc, char** argv)
 {
     extern int optind;
     char filename[256], *exec;
-    int i, j, k, n, rank, nprocs, verbose=1, err, nerrs=0;
+    int i, j, k, n, rank, nprocs, err, nerrs=0;
     int ncid, cmode, varid[4], dimid[2], nreqs, reqs[4], sts[4];
     unsigned int *buffer[4];
     int num_segs[4], req_lens[4];
@@ -168,6 +202,8 @@ int main(int argc, char** argv)
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
     exec = argv[0];
 
+    verbose = 1;
+
     /* get command-line arguments */
     while ((i = getopt(argc, argv, "hq")) != EOF)
         switch(i) {
@@ -203,6 +239,10 @@ int main(int argc, char** argv)
     err = ncmpi_def_var(ncid, "var1", NC_UINT, NDIMS, dimid, &varid[1]); ERR
     err = ncmpi_def_var(ncid, "var2", NC_UINT, NDIMS, dimid, &varid[2]); ERR
     err = ncmpi_def_var(ncid, "var3", NC_UINT, NDIMS, dimid, &varid[3]); ERR
+    if (nprocs < 4) { /* need 4 processes to fill the variables */
+        err = ncmpi_set_fill(ncid, NC_FILL, NULL);
+        ERR
+    }
     err = ncmpi_enddef(ncid); ERR
 
     /* allocate space for starts and counts */
@@ -340,15 +380,7 @@ int main(int argc, char** argv)
     free_3D(starts);
     free_3D(counts);
 
-    /* check if there is any PnetCDF internal malloc residue */
-    MPI_Offset malloc_size, sum_size;
-    err = ncmpi_inq_malloc_size(&malloc_size);
-    if (err == NC_NOERR) {
-        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
-        if (rank == 0 && sum_size > 0)
-            printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
-                   sum_size);
-    }
+    nerrs += pnetcdf_check_mem_usage(MPI_COMM_WORLD);
 
     MPI_Finalize();
     return (nerrs > 0);
