@@ -7,9 +7,9 @@
 /* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * This example shows how to use ncmpi_put_vara_int_all() to write a 2D
- * 4-byte integer array in parallel. It first defines a netCDF variable of
- * size global_nx * global_ny where
+ * This example shows how to use ncmpi_put_vara_int_all() to write a 2D 4-byte
+ * integer array in parallel. It first defines a netCDF variable of size
+ * global_ny * global_nx where
  *    global_ny == NY and
  *    global_nx == (NX * number of MPI processes).
  * The data partitioning pattern is a column-wise partitioning across all
@@ -19,7 +19,7 @@
  *        mpicc -O2 put_vara.c -o put_vara -lpnetcdf
  *
  * Example commands for MPI run and outputs from running ncmpidump on the
- * NC file produced by this example program:
+ * output netCDF file produced by this example program:
  *
  *    % mpiexec -n 4 ./put_vara /pvfs2/wkliao/testfile.nc
  *
@@ -33,9 +33,9 @@
  *            int var(y, x) ;
  *                var:str_att_name = "example attribute of type text." ;
  *                var:float_att_name = 0.f, 1.f, 2.f, 3.f, 4.f, 5.f, 6.f, 7.f ;
- *                var:int64_att_name = 10000000000L ;
+ *                var:short_att_name = 1000s ;
  *    // global attributes:
- *                :history = "Wed Apr 30 11:18:58 2014\n",
+ *                :history = "Mon Aug 13 21:27:48 2018" ;
  *       "" ;
  *    data:
  *
@@ -64,39 +64,146 @@
 #define NY 10
 #define NX 4
 
-#define ERR {if(err!=NC_NOERR){printf("Error at line %d in %s: %s\n", __LINE__,__FILE__, ncmpi_strerror(err));nerrs++;}}
+static int verbose;
+
+#define ERR {if(err!=NC_NOERR){printf("Error at %s:%d : %s\n", __FILE__,__LINE__, ncmpi_strerror(err));nerrs++;}}
 
 static void
 usage(char *argv0)
 {
     char *help =
-    "Usage: %s [-h] | [-q] [file_name]\n"
+    "Usage: %s [-h] | [-q] [-k format] [file_name]\n"
     "       [-h] Print help\n"
     "       [-q] Quiet mode (reports when fail)\n"
+    "       [-k format] file format: 1 for CDF-1, 2 for CDF-2, 3 for NetCDF4,\n"
+    "                                4 for NetCDF4 classic model, 5 for CDF-5\n"
     "       [filename] output netCDF file name\n";
     fprintf(stderr, help, argv0);
 }
 
-int main(int argc, char** argv)
+/*----< pnetcdf_check_mem_usage() >------------------------------------------*/
+/* check PnetCDF library internal memory usage */
+static int
+pnetcdf_check_mem_usage(MPI_Comm comm)
 {
-    extern int optind;
-    char filename[256];
-    int i, j, rank, nprocs, verbose=1, err, nerrs=0;
-    int ncid, cmode, varid, dimid[2], buf[NY][NX];
+    int err, nerrs=0, rank;
+    MPI_Offset malloc_size, sum_size;
+
+    MPI_Comm_rank(comm, &rank);
+
+    /* print info about PnetCDF internal malloc usage */
+    err = ncmpi_inq_malloc_max_size(&malloc_size);
+    if (err == NC_NOERR) {
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && verbose)
+            printf("maximum heap memory allocated by PnetCDF internally is %lld bytes\n",
+                   sum_size);
+
+        /* check if there is any PnetCDF internal malloc residue */
+        err = ncmpi_inq_malloc_size(&malloc_size);
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && sum_size > 0)
+            printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
+                   sum_size);
+    }
+    else if (err != NC_ENOTENABLED) {
+        printf("Error at %s:%d: %s\n", __FILE__,__LINE__,ncmpi_strerror(err));
+        nerrs++;
+    }
+    return nerrs;
+}
+
+/*----< pnetcdf_io() >-------------------------------------------------------*/
+static int
+pnetcdf_io(MPI_Comm comm, char *filename, int cmode)
+{
+    int i, j, rank, nprocs, err, nerrs=0;
+    int ncid, varid, dimid[2], buf[NY][NX];
     char str_att[128];
     float float_att[100];
     MPI_Offset  global_ny, global_nx;
     MPI_Offset start[2], count[2];
 
+    MPI_Comm_rank(comm, &rank);
+    MPI_Comm_size(comm, &nprocs);
+
+    /* create a new file for writing ----------------------------------------*/
+    cmode |= NC_CLOBBER;
+    err = ncmpi_create(comm, filename, cmode, MPI_INFO_NULL, &ncid); ERR
+
+    /* the global array is NY * (NX * nprocs) */
+    global_ny = NY;
+    global_nx = NX * nprocs;
+
+    for (i=0; i<NY; i++)
+        for (j=0; j<NX; j++)
+             buf[i][j] = rank;
+
+    /* add a global attribute: a time stamp at rank 0 */
+    time_t ltime = time(NULL); /* get the current calendar time */
+    asctime_r(localtime(&ltime), str_att);
+    sprintf(str_att, "Mon Aug 13 21:27:48 2018");
+
+    /* make sure the time string are consistent among all processes */
+    MPI_Bcast(str_att, sizeof(str_att), MPI_CHAR, 0, comm);
+
+    err = ncmpi_put_att_text(ncid, NC_GLOBAL, "history", strlen(str_att),
+                             &str_att[0]); ERR
+
+    /* define dimensions x and y */
+    err = ncmpi_def_dim(ncid, "Y", global_ny, &dimid[0]); ERR
+    err = ncmpi_def_dim(ncid, "X", global_nx, &dimid[1]); ERR
+
+    /* define a 2D variable of integer type */
+    err = ncmpi_def_var(ncid, "var", NC_INT, 2, dimid, &varid); ERR
+
+    /* add attributes to the variable */
+    strcpy(str_att, "example attribute of type text.");
+    err = ncmpi_put_att_text(ncid, varid, "str_att_name", strlen(str_att),
+                             &str_att[0]); ERR
+
+    for (i=0; i<8; i++) float_att[i] = i;
+    err = ncmpi_put_att_float(ncid, varid, "float_att_name", NC_FLOAT, 8,
+                              &float_att[0]); ERR
+    short short_att=1000;
+    err = ncmpi_put_att_short(ncid, varid, "short_att_name", NC_SHORT, 1,
+                              &short_att); ERR
+
+    /* do not forget to exit define mode */
+    err = ncmpi_enddef(ncid); ERR
+
+    /* now we are in data mode */
+    start[0] = 0;
+    start[1] = NX * rank;
+    count[0] = NY;
+    count[1] = NX;
+
+    err = ncmpi_put_vara_int_all(ncid, varid, start, count, &buf[0][0]); ERR
+
+    err = ncmpi_close(ncid); ERR
+
+    return nerrs;
+}
+
+int main(int argc, char** argv)
+{
+    extern int optind;
+    extern char *optarg;
+    char filename[256];
+    int i, rank, kind=0, cmode=0, nerrs=0;
+
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
+
+    verbose = 1;
 
     /* get command-line arguments */
-    while ((i = getopt(argc, argv, "hq")) != EOF)
+    while ((i = getopt(argc, argv, "hqk:")) != EOF)
         switch(i) {
             case 'q': verbose = 0;
                       break;
+            case 'k': kind = atoi(optarg);
+                      break;
             case 'h':
             default:  if (rank==0) usage(argv[0]);
                       MPI_Finalize();
@@ -109,81 +216,25 @@ int main(int argc, char** argv)
 
     if (verbose && rank == 0) printf("%s: example of using put_vara APIs\n",__FILE__);
 
-    /* create a new file for writing ----------------------------------------*/
-    cmode = NC_CLOBBER | NC_64BIT_DATA;
-    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
-    ERR
-
-    /* the global array is NY * (NX * nprocs) */
-    global_ny = NY;
-    global_nx = NX * nprocs;
-
-    for (i=0; i<NY; i++)
-        for (j=0; j<NX; j++)
-             buf[i][j] = rank;
-
-    /* add a global attribute: a time stamp at rank 0 */
-    time_t ltime = time(NULL); /* get the current calendar time */
-    asctime_r(localtime(&ltime), str_att);
-
-    /* make sure the time string are consistent among all processes */
-    MPI_Bcast(str_att, strlen(str_att), MPI_CHAR, 0, MPI_COMM_WORLD);
-
-    err = ncmpi_put_att_text(ncid, NC_GLOBAL, "history", strlen(str_att),
-                             &str_att[0]);
-    ERR
-
-    /* define dimensions x and y */
-    err = ncmpi_def_dim(ncid, "Y", global_ny, &dimid[0]);
-    ERR
-    err = ncmpi_def_dim(ncid, "X", global_nx, &dimid[1]);
-    ERR
-
-    /* define a 2D variable of integer type */
-    err = ncmpi_def_var(ncid, "var", NC_INT, 2, dimid, &varid);
-    ERR
-
-    /* add attributes to the variable */
-    strcpy(str_att, "example attribute of type text.");
-    err = ncmpi_put_att_text(ncid, varid, "str_att_name", strlen(str_att),
-                             &str_att[0]);
-    ERR
-
-    for (i=0; i<8; i++) float_att[i] = i;
-    err = ncmpi_put_att_float(ncid, varid, "float_att_name", NC_FLOAT, 8,
-                              &float_att[0]);
-    ERR
-    long long int64_att=10000000000LL;
-    err = ncmpi_put_att_longlong(ncid, varid, "int64_att_name", NC_INT64, 1,
-                              &int64_att);
-    ERR
-
-    /* do not forget to exit define mode */
-    err = ncmpi_enddef(ncid);
-    ERR
-
-    /* now we are in data mode */
-    start[0] = 0;
-    start[1] = NX * rank;
-    count[0] = NY;
-    count[1] = NX;
-
-    err = ncmpi_put_vara_int_all(ncid, varid, start, count, &buf[0][0]);
-    ERR
-
-    err = ncmpi_close(ncid);
-    ERR
-
-    /* check if there is any PnetCDF internal malloc residue */
-    MPI_Offset malloc_size, sum_size;
-    err = ncmpi_inq_malloc_size(&malloc_size);
-    if (err == NC_NOERR) {
-        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
-        if (rank == 0 && sum_size > 0)
-            printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
-                   sum_size);
+    switch (kind) {
+        case(2): cmode = NC_64BIT_OFFSET;             break;
+        case(3): cmode = NC_NETCDF4;                  break;
+        case(4): cmode = NC_NETCDF4|NC_CLASSIC_MODEL; break;
+        case(5): cmode = NC_64BIT_DATA;               break;
+        default: cmode = 0;
     }
 
+#ifndef PNETCDF_DRIVER_NETCDF4
+    /* netcdf4 driver is not enabled, skip */
+    if (kind == 3 || kind == 4) {
+        MPI_Finalize();
+        return 0;
+    }
+#endif
+    nerrs += pnetcdf_io(MPI_COMM_WORLD, filename, cmode);
+
+    nerrs += pnetcdf_check_mem_usage(MPI_COMM_WORLD);
+
     MPI_Finalize();
     return (nerrs > 0);
 }
