@@ -28,7 +28,7 @@
 int ncbbio_log_create(NC_bb* ncbbp,
                       __attribute__((unused)) MPI_Info info)
 {
-    int i, rank, np, err, flag, masterrank, procname_len;
+    int rank, np, err, flag, masterrank, procname_len;
     char logbase[NC_LOG_MAX_PATH], basename[NC_LOG_MAX_PATH];
     char procname[MPI_MAX_PROCESSOR_NAME];
     char *abspath, *fname, *path, *fdir = NULL;
@@ -73,32 +73,20 @@ int ncbbio_log_create(NC_bb* ncbbp,
 
     /* Read environment variable for burst buffer path */
 
-    /* Remove romio driver specifier form the beginning of path */
-    path = strchr(ncbbp->path, ':');
-    if (path == NULL) {
-        /* No driver specifier, use full path */
-        path = ncbbp->path;
-    }
-    else {
-        /* Skip until after the first ':' */
-        path += 1;
-    }
+    /* remove the file system type prefix name if there is any.  For example,
+     * when ncbbp->path = "lustre:/home/foo/testfile.nc", remove "lustre:" to
+     * make path pointing to "/home/foo/testfile.nc", so it can be used in
+     * POSIX realpath() below
+     */
+    path = ncmpii_remove_file_system_type_prefix(ncbbp->path);
 
     /* Determine log base */
     if (ncbbp->logbase[0] != '\0') {
-        /* We don't need driver specifier in logbase as well */
-        logbasep = strchr(ncbbp->logbase, ':');
-        if (logbasep == NULL) {
-            /* No driver specifier, use full path */
-            logbasep = ncbbp->logbase;
-        }
-        else {
-            /* Skip until after the first ':' */
-            logbasep += 1;
-        }
+        /* remove the file system type prefix name if there is any */
+        logbasep = ncmpii_remove_file_system_type_prefix(ncbbp->logbase);
     }
     else {
-        i = strlen(path);
+        size_t i = strlen(path);
         fdir = (char*)NCI_Malloc((i + 1) * sizeof(char));
         strncpy(fdir, path, i + 1);
         /* Search for first '\' from the back */
@@ -268,8 +256,9 @@ int ncbbio_log_create(NC_bb* ncbbp,
     headerp->entry_begin = ncbbp->metadata.nused;
     headerp->basenamelen = strlen(basename);
 
-    /* Process anme */
-    *(int*)((char*)headerp->basename + headerp->basenamelen + 1) = procname_len;
+    /* Process name */
+    memcpy((char*)headerp->basename + headerp->basenamelen + 1,
+           &procname_len, sizeof(int));
     strncpy((char*)headerp->basename + headerp->basenamelen + 5,
             procname, procname_len + 1);
 
@@ -373,7 +362,7 @@ int ncbbio_log_enddef(NC_bb *ncbbp)
 int ncbbio_log_close(NC_bb *ncbbp,
                      int    replay)
 {
-    int err;
+    int err, status=NC_NOERR;
     NC_bb_metadataheader* headerp;
 
 #ifdef PNETCDF_PROFILING
@@ -402,19 +391,18 @@ int ncbbio_log_close(NC_bb *ncbbp,
     /* If log file is created, flush the log */
     if (ncbbp->metalog_fd != NULL) {
         /* Commit to CDF file */
-        if (replay && (headerp->num_entries > 0 || !(fIsSet(ncbbp->flag, NC_MODE_INDEP)))) {
-            ncbbio_log_flush_core(ncbbp);
+        if (replay && (headerp->num_entries > 0 ||
+            !(fIsSet(ncbbp->flag, NC_MODE_INDEP)))) {
+            status = ncbbio_log_flush_core(ncbbp);
         }
 
-        /* Close log file */
+        /* Close metadata log file */
         err = ncbbio_sharedfile_close(ncbbp->metalog_fd);
-        if (err != NC_NOERR) {
-            return err;
-        }
+        if (status == NC_NOERR) status = err;
+
+        /* Close data log file */
         err = ncbbio_sharedfile_close(ncbbp->datalog_fd);
-        if (err != NC_NOERR) {
-            return err;
-        }
+        if (status == NC_NOERR) status = err;
 
         /* Delete log files if delete flag is set */
         if (ncbbp->hints & NC_LOG_HINT_DEL_ON_CLOSE) {
@@ -495,7 +483,7 @@ int ncbbio_log_close(NC_bb *ncbbp,
 #endif
 #endif
 
-    return NC_NOERR;
+    return status;
 }
 
 /*
