@@ -4,8 +4,8 @@
  */
 /* $Id$ */
 
-#ifndef _NC_H
-#define _NC_H
+#ifndef H_NC
+#define H_NC
 
 /*
  * netcdf library 'private' data structures, objects and interfaces
@@ -407,6 +407,8 @@ struct NC {
     MPI_Offset    get_size;  /* amount of reads  committed so far in bytes */
 
     MPI_Comm      comm;           /* MPI communicator */
+    int           rank;           /* MPI rank of this process */
+    int           nprocs;         /* number of MPI processes */
     MPI_Info      mpiinfo;        /* used MPI info object */
     MPI_File      collective_fh;  /* file handle for collective mode */
     MPI_File      independent_fh; /* file handle for independent mode */
@@ -433,6 +435,20 @@ struct NC {
 
     char         *path;     /* file name */
     struct NC    *old;      /* contains the previous NC during redef. */
+
+    /* Below are used for intra-node aggregation */
+    int  num_aggrs_per_node; /* number of aggregators per compute node. Set
+                                through a user hint. 0 to disable the
+                                intra-node aggregation, -1 to let PnetCDF to
+                                decide. This value must be the same among all
+                                processes.
+                              */
+    int  my_aggr;            /* rank ID of my aggregator */
+    int  num_nonaggrs;       /* number of non-aggregators assigned */
+    int *nonaggr_ranks;      /* ranks of assigned non-aggregators */
+#ifdef PNETCDF_PROFILING
+    double aggr_time;
+#endif
 };
 
 #define NC_readonly(ncp)   fIsSet((ncp)->flags, NC_MODE_RDONLY)
@@ -474,7 +490,7 @@ typedef struct bufferinfo {
     int         chunk;    /* chunk size for reading the header */
     int         version;  /* 1, 2, and 5 for CDF-1, 2, and 5 respectively */
     int         safe_mode;/* 0: disabled, 1: enabled */
-    int         rw_mode;  /* 0: independent, 1: collective */
+    int         coll_mode;/* 0: independent, 1: collective */
     char       *base;     /* beginning of read/write buffer */
     char       *pos;      /* current position in buffer */
     char       *end;      /* end position of buffer */
@@ -629,4 +645,17 @@ ncmpio_read_write(NC *ncp, int rw_flag, int coll_indep, MPI_Offset offset,
                   MPI_Offset buf_count, MPI_Datatype buf_type, void *buf,
                   int buftype_is_contig);
 
-#endif /* _NC_H */
+/* Begin defined in ncmpio_intranode.c --------------------------------------*/
+extern int
+ncmpio_intra_node_aggr_init(NC *ncp);
+
+extern int
+ncmpio_intra_node_aggregation_nreqs(NC *ncp, int mode, int num_reqs,
+                                    NC_req *put_list, MPI_Offset newnumrecs);
+extern int
+ncmpio_intra_node_aggregation(NC *ncp, int mode, NC_var *varp,
+                              const MPI_Offset *start, const MPI_Offset *count,
+                              const MPI_Offset *stride, MPI_Offset bufCount,
+                              MPI_Datatype bufType, void *buf);
+
+#endif /* H_NC */
