@@ -48,10 +48,11 @@ ncmpio_free_NC(NC *ncp)
      */
     if (ncp->mpiinfo != MPI_INFO_NULL) MPI_Info_free(&ncp->mpiinfo);
 
-    if (ncp->get_list != NULL) NCI_Free(ncp->get_list);
-    if (ncp->put_list != NULL) NCI_Free(ncp->put_list);
-    if (ncp->abuf     != NULL) NCI_Free(ncp->abuf);
-    if (ncp->path     != NULL) NCI_Free(ncp->path);
+    if (ncp->get_list      != NULL) NCI_Free(ncp->get_list);
+    if (ncp->put_list      != NULL) NCI_Free(ncp->put_list);
+    if (ncp->abuf          != NULL) NCI_Free(ncp->abuf);
+    if (ncp->path          != NULL) NCI_Free(ncp->path);
+    if (ncp->nonaggr_ranks != NULL) NCI_Free(ncp->nonaggr_ranks);
 
     NCI_Free(ncp);
 }
@@ -69,7 +70,7 @@ ncmpio_close_files(NC *ncp, int doUnlink) {
             return ncmpii_error_mpi2nc(mpireturn, "MPI_File_close");
     }
 
-    if (ncp->collective_fh != MPI_FILE_NULL) {
+    if (ncp->nprocs > 1 && ncp->collective_fh != MPI_FILE_NULL) {
         TRACE_IO(MPI_File_close)(&ncp->collective_fh);
         if (mpireturn != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(mpireturn, "MPI_File_close");
@@ -78,9 +79,13 @@ ncmpio_close_files(NC *ncp, int doUnlink) {
     if (doUnlink) {
         /* called from ncmpi_abort, if the file is being created and is still
          * in define mode, the file is deleted */
-        TRACE_IO(MPI_File_delete)((char *)ncp->path, ncp->mpiinfo);
-        if (mpireturn != MPI_SUCCESS)
-            return ncmpii_error_mpi2nc(mpireturn, "MPI_File_delete");
+        if (ncp->rank == 0) {
+            TRACE_IO(MPI_File_delete)((char *)ncp->path, ncp->mpiinfo);
+            if (mpireturn != MPI_SUCCESS)
+                return ncmpii_error_mpi2nc(mpireturn, "MPI_File_delete");
+        }
+        if (ncp->nprocs > 1)
+            MPI_Barrier(ncp->comm);
     }
     return NC_NOERR;
 }
@@ -140,17 +145,13 @@ ncmpio_close(void *ncdp)
     }
 #else
     if (ncp->numLeadGetReqs > 0) {
-        int rank;
-        MPI_Comm_rank(ncp->comm, &rank);
-        printf("PnetCDF warning: %d nonblocking get requests still pending on process %d. Cancelling ...\n",ncp->numLeadGetReqs,rank);
+        printf("PnetCDF warning: %d nonblocking get requests still pending on process %d. Cancelling ...\n",ncp->numLeadGetReqs,ncp->rank);
         err = ncmpio_cancel(ncp, NC_GET_REQ_ALL, NULL, NULL);
         if (status == NC_NOERR) status = err;
         if (status == NC_NOERR) status = NC_EPENDING;
     }
     if (ncp->numLeadPutReqs > 0) {
-        int rank;
-        MPI_Comm_rank(ncp->comm, &rank);
-        printf("PnetCDF warning: %d nonblocking put requests still pending on process %d. Cancelling ...\n",ncp->numLeadPutReqs,rank);
+        printf("PnetCDF warning: %d nonblocking put requests still pending on process %d. Cancelling ...\n",ncp->numLeadPutReqs,ncp->rank);
         err = ncmpio_cancel(ncp, NC_PUT_REQ_ALL, NULL, NULL);
         if (status == NC_NOERR) status = err;
         if (status == NC_NOERR) status = NC_EPENDING;
@@ -163,13 +164,10 @@ ncmpio_close(void *ncdp)
 
     /* file is open for write and no variable has been defined */
     if (!NC_readonly(ncp) && ncp->vars.ndefined == 0) {
-        int rank;
-
         /* wait until all processes close the file */
-        MPI_Barrier(ncp->comm);
+        if (ncp->nprocs > 1) MPI_Barrier(ncp->comm);
 
-        MPI_Comm_rank(ncp->comm, &rank);
-        if (rank == 0) {
+        if (ncp->rank == 0) {
             /* ignore all errors, as unexpected file size if not a fatal error */
 #ifdef HAVE_TRUNCATE
             /* when calling POSIX I/O, remove file type prefix from file name */
@@ -192,17 +190,29 @@ ncmpio_close(void *ncdp)
             if (mpireturn == MPI_SUCCESS) {
                 /* obtain file size */
                 MPI_Offset *file_size;
-                MPI_File_seek(fh, 0, MPI_SEEK_END);
-                MPI_File_get_position(fh, &file_size);
+                mpireturn = MPI_File_seek(fh, 0, MPI_SEEK_END);
+                if (mpireturn != MPI_SUCCESS) {
+                    err = ncmpii_error_mpi2nc(mpireturn,"MPI_File_seek");
+                    if (status == NC_NOERR) status = err;
+                }
+                mpireturn = MPI_File_get_position(fh, &file_size);
+                if (mpireturn != MPI_SUCCESS) {
+                    err = ncmpii_error_mpi2nc(mpireturn,"MPI_File_get_position");
+                    if (status == NC_NOERR) status = err;
+                }
                 /* truncate file size to header size, if larger than header */
-                if (file_size > ncp->xsz} {
+                if (file_size > ncp->xsz) {
                     mpireturn = MPI_File_set_size(fh, ncp->xsz);
                     if (mpireturn != MPI_SUCCESS) {
                         err = ncmpii_error_mpi2nc(mpireturn,"MPI_File_set_size");
                         if (status == NC_NOERR) status = err;
                     }
                 }
-                MPI_File_close(&fh);
+                mpireturn = MPI_File_close(&fh);
+                if (mpireturn != MPI_SUCCESS) {
+                    err = ncmpii_error_mpi2nc(mpireturn,"MPI_File_close");
+                    if (status == NC_NOERR) status = err;
+                }
             }
             else {
                 err = ncmpii_error_mpi2nc(mpireturn,"MPI_File_open");
@@ -210,7 +220,7 @@ ncmpio_close(void *ncdp)
             }
 #endif
         }
-        MPI_Barrier(ncp->comm);
+        if (ncp->nprocs > 1) MPI_Barrier(ncp->comm);
     }
 
     /* free up space occupied by the header metadata */
