@@ -113,10 +113,12 @@ ncmpio_open(MPI_Comm     comm,
 
     ncp->iomode         = omode;
     ncp->comm           = comm;  /* reuse comm duplicated in dispatch layer */
+    MPI_Comm_rank(comm, &ncp->rank);
+    MPI_Comm_size(comm, &ncp->nprocs);
     ncp->mpiinfo        = info_used; /* is not MPI_INFO_NULL */
     ncp->mpiomode       = mpiomode;
     ncp->collective_fh  = fh;
-    ncp->independent_fh = MPI_FILE_NULL;
+    ncp->independent_fh = (ncp->nprocs > 1) ? MPI_FILE_NULL : fh;
     ncp->path = (char*) NCI_Malloc(strlen(path) + 1);
     strcpy(ncp->path, path);
 
@@ -189,6 +191,19 @@ ncmpio_open(MPI_Comm     comm,
         ncp->vars.value[i]->attrs.hash_size = ncp->hash_size_attr;
 #endif
 
+    /* determine whether to enable intra-node aggregation and set up all
+     * intra-node aggregation metadata.
+     * ncp->num_aggrs_per_node = 0, or non-zero indicates whether this feature
+     *     is enabled globally for all processes.
+     * ncp->my_aggr = -1 or >= 0 indicates whether aggregation is effectively
+     *     enabled for the aggregation group of this process.
+     */
+    ncp->my_aggr = -1;
+    if (ncp->num_aggrs_per_node != 0) {
+        err = ncmpio_intra_node_aggr_init(ncp);
+        if (err != NC_NOERR) return err;
+    }
+
     *ncpp = (void*)ncp;
 
     return status;
