@@ -52,10 +52,12 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->env_h_align = strtoll(value, NULL, 10);
             if (errno != 0) ncp->env_h_align = 0;
             else if (ncp->env_h_align < 0) ncp->env_h_align = 0;
-            sprintf(value, "%lld", ncp->env_h_align);
         }
     }
-    if (!flag) sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    if (ncp->env_h_align == 0)
+        sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    else
+        sprintf(value, "%lld", ncp->env_h_align);
     MPI_Info_set(info_used, "nc_header_align_size", value);
 
     ncp->env_v_align = 0;
@@ -68,10 +70,12 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->env_v_align = strtoll(value, NULL, 10);
             if (errno != 0) ncp->env_v_align = 0;
             else if (ncp->env_v_align < 0) ncp->env_v_align = 0;
-            sprintf(value, "%lld", ncp->env_v_align);
         }
     }
-    if (!flag) sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    if (ncp->env_v_align == 0)
+        sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    else
+        sprintf(value, "%lld", ncp->env_v_align);
     MPI_Info_set(info_used, "nc_var_align_size", value);
 
     ncp->env_r_align = 0;
@@ -84,12 +88,15 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->env_r_align = strtoll(value, NULL, 10);
             if (errno != 0) ncp->env_r_align = 0;
             else if (ncp->env_r_align < 0) ncp->env_r_align = 0;
-            sprintf(value, "%lld", ncp->env_r_align);
         }
     }
-    if (!flag) sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    if (ncp->env_r_align == 0)
+        sprintf(value, "%d", FILE_ALIGNMENT_DEFAULT);
+    else
+        sprintf(value, "%lld", ncp->env_r_align);
     MPI_Info_set(info_used, "nc_record_align_size", value);
 
+    ncp->chunk = PNC_DEFAULT_CHUNKSIZE;
     if (user_info != MPI_INFO_NULL) {
         /* header reading chunk size */
         MPI_Info_get(user_info, "nc_header_read_chunk_size", MPI_MAX_INFO_VAL-1,
@@ -103,12 +110,12 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
                 ncp->chunk = 0;
             else if (chunk > NC_MAX_INT) /* limit to NC_MAX_INT */
                 ncp->chunk = NC_MAX_INT;
-            sprintf(value, "%d", ncp->chunk);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_DEFAULT_CHUNKSIZE);
+    sprintf(value, "%d", ncp->chunk);
     MPI_Info_set(info_used, "nc_header_read_chunk_size", value);
 
+    strcpy(value, "auto");
     if (user_info != MPI_INFO_NULL) {
         /* setting in-place byte swap (matters only for Little Endian) */
         MPI_Info_get(user_info, "nc_in_place_swap", MPI_MAX_INFO_VAL-1, value, &flag);
@@ -127,9 +134,9 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             }
         }
     }
-    if (!flag) strcpy(value, "auto");
     MPI_Info_set(info_used, "nc_in_place_swap", value);
 
+    ncp->ibuf_size = PNC_DEFAULT_IBUF_SIZE;
     if (user_info != MPI_INFO_NULL) {
 	/* temporal buffer size used to pack noncontiguous aggregated user
          * buffers when calling ncmpi_wait/wait_all, Default 16 MiB
@@ -141,13 +148,13 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             errno = 0;  /* errno must set to zero before calling strtoll */
             ibuf_size = strtoll(value, NULL, 10);
             if (errno == 0 && ncp->ibuf_size > 0) ncp->ibuf_size = ibuf_size;
-            sprintf(value, "%lld", ncp->ibuf_size);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_DEFAULT_IBUF_SIZE);
+    sprintf(value, "%lld", ncp->ibuf_size);
     MPI_Info_set(info_used, "nc_ibuf_size", value);
 
 #ifdef ENABLE_SUBFILING
+    ncp->subfile_mode = 0;
     if (user_info != MPI_INFO_NULL) {
         MPI_Info_get(user_info, "pnetcdf_subfiling", MPI_MAX_INFO_VAL-1,
                      value, &flag);
@@ -156,9 +163,12 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
                 ncp->subfile_mode = 1;
         }
     }
-    if (!flag) strcpy(value, "disable");
-    MPI_Info_set(info_used, "pnetcdf_subfiling", value);
+    if (ncp->subfile_mode)
+        MPI_Info_set(info_used, "pnetcdf_subfiling", "enable");
+    else
+        MPI_Info_set(info_used, "pnetcdf_subfiling", "disable");
 
+    ncp->num_subfiles = 0;
     if (user_info != MPI_INFO_NULL) {
         MPI_Info_get(user_info, "nc_num_subfiles", MPI_MAX_INFO_VAL-1,
                      value, &flag);
@@ -167,10 +177,9 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->num_subfiles = atoi(value);
             if (errno != 0) ncp->num_subfiles = 0;
             else if (ncp->num_subfiles < 0) ncp->num_subfiles = 0;
-            sprintf(value, "%d", ncp->num_subfiles);
         }
     }
-    if (!flag) strcpy(value, "0");
+    sprintf(value, "%d", ncp->num_subfiles);
     MPI_Info_set(info_used, "nc_num_subfiles", value);
 
     if (ncp->subfile_mode == 0) ncp->num_subfiles = 0;
@@ -202,10 +211,9 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->dims.hash_size = atoi(value);
             if (errno != 0 || ncp->dims.hash_size < 0)
                 ncp->dims.hash_size = PNC_HSIZE_DIM;
-            sprintf(value, "%d", ncp->dims.hash_size);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_HSIZE_DIM);
+    sprintf(value, "%d", ncp->dims.hash_size);
     MPI_Info_set(info_used, "nc_hash_size_dim", value);
 
     ncp->vars.hash_size = PNC_HSIZE_VAR;
@@ -218,10 +226,9 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->vars.hash_size = atoi(value);
             if (errno != 0 || ncp->vars.hash_size < 0)
                 ncp->vars.hash_size = PNC_HSIZE_VAR;
-            sprintf(value, "%d", ncp->vars.hash_size);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_HSIZE_VAR);
+    sprintf(value, "%d", ncp->vars.hash_size);
     MPI_Info_set(info_used, "nc_hash_size_var", value);
 
     ncp->attrs.hash_size = PNC_HSIZE_GATTR;
@@ -234,10 +241,9 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->attrs.hash_size = atoi(value);
             if (errno != 0 || ncp->attrs.hash_size < 0)
                 ncp->attrs.hash_size = PNC_HSIZE_GATTR;
-            sprintf(value, "%d", ncp->attrs.hash_size);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_HSIZE_GATTR);
+    sprintf(value, "%d", ncp->attrs.hash_size);
     MPI_Info_set(info_used, "nc_hash_size_gattr", value);
 
     ncp->hash_size_attr = PNC_HSIZE_VATTR;
@@ -250,11 +256,25 @@ void ncmpio_set_pnetcdf_hints(NC *ncp,
             ncp->hash_size_attr = atoi(value);
             if (errno != 0 || ncp->hash_size_attr < 0)
                 ncp->hash_size_attr = PNC_HSIZE_VATTR;
-            sprintf(value, "%d", ncp->hash_size_attr);
         }
     }
-    if (!flag) sprintf(value, "%d", PNC_HSIZE_VATTR);
+    sprintf(value, "%d", ncp->hash_size_attr);
     MPI_Info_set(info_used, "nc_hash_size_vattr", value);
+
+    ncp->num_aggrs_per_node = 0;
+    if (user_info != MPI_INFO_NULL) {
+        /* Hash table size for non-global attributes */
+        MPI_Info_get(user_info, "nc_num_aggrs_per_node", MPI_MAX_INFO_VAL-1,
+                     value, &flag);
+        if (flag) {
+            errno = 0;  /* errno must set to zero before calling atoi */
+            ncp->num_aggrs_per_node = atoi(value);
+            if (errno != 0 || ncp->num_aggrs_per_node < 0)
+                ncp->num_aggrs_per_node = 0;
+        }
+    }
+    sprintf(value, "%d", ncp->num_aggrs_per_node);
+    MPI_Info_set(info_used, "nc_num_aggrs_per_node", value);
 }
 
 /*----< ncmpio_first_offset() >-----------------------------------------------*/
@@ -443,7 +463,7 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
                  void         *buf,    /* user buffer */
                  void         *xbuf)   /* already allocated, in external type */
 {
-    int err=NC_NOERR, free_lbuf=0, free_cbuf=0;
+    int err=NC_NOERR, mpireturn, free_lbuf=0, free_cbuf=0;
     void *lbuf=NULL, *cbuf=NULL;
     MPI_Offset ibuf_size;
 
@@ -469,16 +489,20 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
             /* pack buf into lbuf based on buftype */
 #ifdef HAVE_MPI_LARGE_COUNT
             MPI_Count position = 0;
-            MPI_Pack_c(buf, (MPI_Count)bufcount, buftype, lbuf,
-                       (MPI_Count)ibuf_size, &position, MPI_COMM_SELF);
+            mpireturn = MPI_Pack_c(buf, (MPI_Count)bufcount, buftype, lbuf,
+                                   (MPI_Count)ibuf_size, &position, MPI_COMM_SELF);
+            if (mpireturn != MPI_SUCCESS)
+                return ncmpii_error_mpi2nc(mpireturn, "MPI_Pack_c");
 #else
             int position = 0;
             if (bufcount > NC_MAX_INT || ibuf_size > NC_MAX_INT) {
                 if (free_lbuf) NCI_Free(lbuf);
                 DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
             }
-            MPI_Pack(buf, (int)bufcount, buftype, lbuf, (int)ibuf_size,
-                     &position, MPI_COMM_SELF);
+            mpireturn = MPI_Pack(buf, (int)bufcount, buftype, lbuf, (int)ibuf_size,
+                                 &position, MPI_COMM_SELF);
+            if (mpireturn != MPI_SUCCESS)
+                return ncmpii_error_mpi2nc(mpireturn, "MPI_Pack");
 #endif
         }
     }
@@ -504,14 +528,18 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         /* pack lbuf to cbuf based on imaptype */
 #ifdef HAVE_MPI_LARGE_COUNT
         MPI_Count position = 0;
-        MPI_Pack_c(lbuf, 1, imaptype, cbuf, (MPI_Count)ibuf_size, &position,
-                   MPI_COMM_SELF);
+        mpireturn = MPI_Pack_c(lbuf, 1, imaptype, cbuf, (MPI_Count)ibuf_size,
+                               &position, MPI_COMM_SELF);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(mpireturn, "MPI_Pack_c");
 #else
         int position = 0;
         if (ibuf_size > NC_MAX_INT)
             DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-        MPI_Pack(lbuf, 1, imaptype, cbuf, (int)ibuf_size, &position,
-                 MPI_COMM_SELF);
+        mpireturn = MPI_Pack(lbuf, 1, imaptype, cbuf, (int)ibuf_size,
+                             &position, MPI_COMM_SELF);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(mpireturn, "MPI_Pack");
 #endif
         MPI_Type_free(&imaptype);
 
@@ -638,7 +666,7 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
                    void         *buf,  /* user buffer */
                    void         *xbuf) /* already allocated, in external type */
 {
-    int err=NC_NOERR, el_size, free_lbuf=0, free_cbuf=0;
+    int err=NC_NOERR, mpireturn, el_size, free_lbuf=0, free_cbuf=0;
     void *lbuf=NULL, *cbuf=NULL;
     MPI_Offset ibuf_size;
 
@@ -737,14 +765,18 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         /* unpack cbuf to lbuf based on imaptype */
 #ifdef HAVE_MPI_LARGE_COUNT
         MPI_Count position = 0;
-        MPI_Unpack_c(cbuf, (MPI_Count)ibuf_size, &position, lbuf, 1, imaptype,
-                     MPI_COMM_SELF);
+        mpireturn = MPI_Unpack_c(cbuf, (MPI_Count)ibuf_size, &position, lbuf,
+                                 1, imaptype, MPI_COMM_SELF);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(mpireturn, "MPI_Unpack_c");
 #else
         int position = 0;
         if (ibuf_size > NC_MAX_INT) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
 
-        MPI_Unpack(cbuf, (int)ibuf_size, &position, lbuf, 1, imaptype,
-                   MPI_COMM_SELF);
+        mpireturn = MPI_Unpack(cbuf, (int)ibuf_size, &position, lbuf,
+                               1, imaptype, MPI_COMM_SELF);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(mpireturn, "MPI_Unpack");
 #endif
         MPI_Type_free(&imaptype);
     }
@@ -754,8 +786,10 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         /* no need unpack when buftype is used in MPI_File_read (lbuf == buf) */
 #ifdef HAVE_MPI_LARGE_COUNT
         MPI_Count position = 0;
-        MPI_Unpack_c(lbuf, (MPI_Count)ibuf_size, &position, buf,
+        mpireturn = MPI_Unpack_c(lbuf, (MPI_Count)ibuf_size, &position, buf,
                      (MPI_Count)bufcount, buftype, MPI_COMM_SELF);
+        if (mpireturn != MPI_SUCCESS)
+            return ncmpii_error_mpi2nc(mpireturn, "MPI_Unpack_c");
 #else
         if (bufcount > NC_MAX_INT) {
             if (err == NC_NOERR)
@@ -765,8 +799,10 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
             int position = 0;
             if (ibuf_size > NC_MAX_INT)
                 DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-            MPI_Unpack(lbuf, (int)ibuf_size, &position, buf, (int)bufcount,
-                       buftype, MPI_COMM_SELF);
+            mpireturn = MPI_Unpack(lbuf, (int)ibuf_size, &position, buf,
+                                   (int)bufcount, buftype, MPI_COMM_SELF);
+            if (mpireturn != MPI_SUCCESS)
+                return ncmpii_error_mpi2nc(mpireturn, "MPI_Unpack");
         }
 #endif
     }
