@@ -121,13 +121,18 @@ getput_vard(NC               *ncp,
 
 #ifdef HAVE_MPI_TYPE_GET_TRUE_EXTENT_C
     /* MPI_Type_get_true_extent_c is introduced in MPI 4.0 */
-    MPI_Type_get_true_extent_c(filetype, &true_lb, &true_extent);
+    mpireturn = MPI_Type_get_true_extent_c(filetype, &true_lb, &true_extent);
 #elif defined(HAVE_MPI_TYPE_GET_TRUE_EXTENT_X)
     /* MPI_Type_get_true_extent_x is introduced in MPI 3.0 */
-    MPI_Type_get_true_extent_x(filetype, &true_lb, &true_extent);
+    mpireturn = MPI_Type_get_true_extent_x(filetype, &true_lb, &true_extent);
 #else
-    MPI_Type_get_true_extent(filetype, &true_lb, &true_extent);
+    mpireturn = MPI_Type_get_true_extent(filetype, &true_lb, &true_extent);
 #endif
+    if (mpireturn != MPI_SUCCESS) {
+        err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_get_true_extent");
+        goto err_check;
+    }
+
     true_ub = true_lb + true_extent;
 
     /* get the corresponding MPI datatype of variable external type */
@@ -154,7 +159,11 @@ getput_vard(NC               *ncp,
     if (buftype == MPI_DATATYPE_NULL) {
         /* In this case, the request size is the same as filetype */
         buftype = etype = xtype;
-        MPI_Type_size(buftype, &el_size);
+        mpireturn = MPI_Type_size(buftype, &el_size);
+        if (mpireturn != MPI_SUCCESS) {
+            err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_size");
+            goto err_check;
+        }
         bufcount = filetype_size / el_size;
         buftype_is_contig = 1;
         bnelems = bufcount;
@@ -185,17 +194,27 @@ getput_vard(NC               *ncp,
     need_swap    = NEED_BYTE_SWAP(varp->xtype, etype);
 
     if (fIsSet(reqMode, NC_REQ_WR)) {
-        int in_place_swap = 0;
+        /* check if in-place byte swap can be enabled */
+        int can_swap_in_place = 1;
         if (need_swap) {
-            if (fIsSet(ncp->flags, NC_MODE_SWAP_ON))
-                in_place_swap = 1;
-            else if (! fIsSet(ncp->flags, NC_MODE_SWAP_OFF)) { /* auto mode */
-                if (filetype_size > NC_BYTE_SWAP_BUFFER_SIZE)
-                    in_place_swap = 1;
+            if (fIsSet(ncp->flags, NC_MODE_SWAP_OFF))
+                /* in-place byte swap is disabled by user through PnetCDF hint
+                 * 'nc_in_place_swap'.
+                 */
+                can_swap_in_place = 0;
+            else if (! fIsSet(ncp->flags, NC_MODE_SWAP_ON)) {
+                /* auto mode, as user does not explicitly enable it */
+                if (filetype_size <= NC_BYTE_SWAP_BUFFER_SIZE)
+                    /* If write amount is small, disable in-place swap.
+                     * This is because the user buffer may be immutable.
+                     * In this case, in-place swap will cause segmentation
+                     * fault. Immutable buffers are usually small.  */
+                    can_swap_in_place = 0;
             }
         }
+
         if (!need_convert &&
-            (!need_swap || (in_place_swap && buftype_is_contig))) {
+            (!need_swap || (can_swap_in_place && buftype_is_contig))) {
             /* reuse buftype, bufcount, buf in later MPI file write */
             xbuf = buf;
             if (need_swap) {
@@ -285,12 +304,12 @@ err_check:
     }
     status = err;
 
-    if (fIsSet(reqMode, NC_REQ_COLL)) {
+    /* when ncp->nprocs == 1, ncp->collective_fh == ncp->independent_fh */
+    fh = ncp->independent_fh;
+    coll_indep = NC_REQ_INDEP;
+    if (ncp->nprocs > 1 && fIsSet(reqMode, NC_REQ_COLL)) {
         fh = ncp->collective_fh;
         coll_indep = NC_REQ_COLL;
-    } else {
-        fh = ncp->independent_fh;
-        coll_indep = NC_REQ_INDEP;
     }
 
     /* set the MPI-IO fileview, this is a collective call */
@@ -346,12 +365,14 @@ err_check:
                 /* new_numrecs may be different among processes.
                  * First, find the max numrecs among all processes.
                  */
-                MPI_Offset max_numrecs;
-                TRACE_COMM(MPI_Allreduce)(&new_numrecs, &max_numrecs, 1,
-                                          MPI_OFFSET, MPI_MAX, ncp->comm);
-                if (mpireturn != MPI_SUCCESS) {
-                    err = ncmpii_error_mpi2nc(mpireturn, "MPI_Allreduce");
-                    if (status == NC_NOERR) status = err;
+                MPI_Offset max_numrecs = new_numrecs;
+                if (ncp->nprocs > 1) {
+                    TRACE_COMM(MPI_Allreduce)(&new_numrecs, &max_numrecs, 1,
+                                              MPI_OFFSET, MPI_MAX, ncp->comm);
+                    if (mpireturn != MPI_SUCCESS) {
+                        err = ncmpii_error_mpi2nc(mpireturn, "MPI_Allreduce");
+                        if (status == NC_NOERR) status = err;
+                    }
                 }
                 /* In collective mode, ncp->numrecs is always sync-ed among
                    processes */
