@@ -65,14 +65,20 @@ ncmpio_getput_zero_req(NC *ncp, int reqMode)
                                 MPI_INFO_NULL);
 
     if (fIsSet(reqMode, NC_REQ_RD)) {
-        TRACE_IO(MPI_File_read_all)(fh, NULL, 0, MPI_BYTE, &mpistatus);
+        if (ncp->nprocs > 1)
+            TRACE_IO(MPI_File_read_all)(fh, NULL, 0, MPI_BYTE, &mpistatus);
+        else
+            TRACE_IO(MPI_File_read)(fh, NULL, 0, MPI_BYTE, &mpistatus);
         if (mpireturn != MPI_SUCCESS) {
             err = ncmpii_error_mpi2nc(mpireturn, "MPI_File_read_all");
             err = (err == NC_EFILE) ? NC_EREAD : err;
             DEBUG_ASSIGN_ERROR(status, err)
         }
     } else { /* write request */
-        TRACE_IO(MPI_File_write_all)(fh, NULL, 0, MPI_BYTE, &mpistatus);
+        if (ncp->nprocs > 1)
+            TRACE_IO(MPI_File_write_all)(fh, NULL, 0, MPI_BYTE, &mpistatus);
+        else
+            TRACE_IO(MPI_File_write)(fh, NULL, 0, MPI_BYTE, &mpistatus);
         if (mpireturn != MPI_SUCCESS) {
             err = ncmpii_error_mpi2nc(mpireturn, "MPI_File_write_all");
             err = (err == NC_EFILE) ? NC_EWRITE : err;
@@ -339,6 +345,7 @@ construct_filetypes(NC           *ncp,
                     MPI_Datatype *filetype)  /* OUT */
 {
     int i, j, err, status=NC_NOERR, all_ftype_contig=1, last_contig_req;
+    int mpireturn;
     MPI_Datatype *ftypes;
 
     if (num_reqs <= 0) { /* for participating collective call */
@@ -463,13 +470,14 @@ construct_filetypes(NC           *ncp,
     else if (num_reqs == 1 && disps[0] == 0) {
         if (ftypes[0] == MPI_BYTE)
             *filetype = MPI_BYTE;
-        else
-            MPI_Type_dup(ftypes[0], filetype);
+        else {
+            mpireturn = MPI_Type_dup(ftypes[0], filetype);
+            if (mpireturn != MPI_SUCCESS)
+                err = ncmpii_error_mpi2nc(mpireturn, "MPI_Type_dup");
+        }
     }
     else { /* if (num_reqs > 1 || (num_reqs == 1 && disps[0] > 0)) */
         /* all ftypes[] created fine, now concatenate all ftypes[] */
-        int mpireturn;
-
         if (all_ftype_contig) {
 #ifdef HAVE_MPI_LARGE_COUNT
             mpireturn = MPI_Type_create_hindexed_c(num_reqs, blocklens, disps,
@@ -567,7 +575,7 @@ construct_buffertypes(NC_lead_req  *lead_list,
 
         MPI_Get_address(reqs[i].xbuf, &ai);
         if (j == 0) a0 = ai;
-        disps[j] = ai - a0;
+        disps[j] = MPI_Aint_diff(ai, a0);
         j++;
     }
     /* update num_reqs to number of valid requests */
@@ -948,15 +956,15 @@ req_commit(NC  *ncp,
     }
 
     /* synchronize request metadata across processes if collective I/O */
-    if (coll_indep == NC_REQ_COLL) {
+    if (coll_indep == NC_REQ_COLL && ncp->nprocs > 1) {
         int mpireturn;
-        MPI_Offset io_req[4], do_io[4];  /* [0]: read [1]: write [2]: error */
-        io_req[0] = num_r_reqs;
-        io_req[1] = num_w_reqs;
-        io_req[2] = -err;   /* all NC errors are negative */
-        io_req[3] = newnumrecs;
-        TRACE_COMM(MPI_Allreduce)(io_req, do_io, 4, MPI_OFFSET, MPI_MAX,
-                                  ncp->comm);
+        MPI_Offset do_io[4];  /* [0]: read [1]: write [2]: error */
+        do_io[0] = num_r_reqs;
+        do_io[1] = num_w_reqs;
+        do_io[2] = -err;   /* all NC errors are negative */
+        do_io[3] = newnumrecs;
+        TRACE_COMM(MPI_Allreduce)(MPI_IN_PLACE, do_io, 4, MPI_OFFSET,
+                                  MPI_MAX, ncp->comm);
         if (mpireturn != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(mpireturn, "MPI_Allreduce");
 
@@ -976,9 +984,17 @@ req_commit(NC  *ncp,
     }
 
     /* carry out writes and reads separately (writes first) */
+
     if (do_write > 0) {
-        err = wait_getput(ncp, num_w_reqs, put_list, NC_REQ_WR, coll_indep,
-                          newnumrecs);
+
+        if (ncp->my_aggr >= 0 && coll_indep == NC_REQ_COLL && ncp->nprocs > 1)
+            /* intra-node write aggregation must be in collective mode */
+            err = ncmpio_intra_node_aggregation_nreqs(ncp, NC_REQ_WR,
+                                                      num_w_reqs, put_list,
+                                                      newnumrecs);
+        else
+            err = wait_getput(ncp, num_w_reqs, put_list, NC_REQ_WR, coll_indep,
+                              newnumrecs);
         put_list = NULL; /* has been freed in wait_getput() */
     }
 
@@ -1250,8 +1266,8 @@ off_compare(const void *a, const void *b)
 static MPI_Offset
 vars_flatten(int          ndim,    /* number of dimensions */
              int          el_size, /* array element size */
-             MPI_Offset  *dimlen,  /* [ndim] dimension lengths */
              MPI_Offset   offset,  /* starting file offset of variable */
+             MPI_Offset  *dimlen,  /* [ndim] dimension lengths */
              MPI_Aint     buf_addr,/* starting buffer address */
              MPI_Offset  *start,   /* [ndim] starts of subarray */
              MPI_Offset  *count,   /* [ndim] counts of subarray */
@@ -1349,6 +1365,9 @@ vars_flatten(int          ndim,    /* number of dimensions */
 }
 
 /*----< merge_requests() >---------------------------------------------------*/
+/* flatten all requests into offset-length pairs, sort them into an increasing
+ * order, and resolve the overlapped offset-length pairs.
+ */
 static int
 merge_requests(NC          *ncp,
                NC_lead_req *lead_list,
@@ -1419,7 +1438,7 @@ merge_requests(NC          *ncp,
 
         /* buf_addr is the buffer address of the first valid request */
         MPI_Get_address(reqs[i].xbuf, &addr);
-        addr -= buf_addr,  /* distance to the buf of first req */
+        addr = MPI_Aint_diff(addr, buf_addr);  /* distance to the buf of first req */
 
         ndims = lead->varp->ndims;
         if (ndims > 0) {
@@ -1449,7 +1468,7 @@ merge_requests(NC          *ncp,
         if (fIsSet(lead->flag, NC_REQ_STRIDE_NULL)) stride = NULL;
 
         /* flatten each request to a list of offset-length pairs */
-        vars_flatten(ndims, lead->varp->xsz, shape, var_begin,
+        vars_flatten(ndims, lead->varp->xsz, var_begin, shape,
                      addr, start, count, stride,
                      &nseg,    /* OUT: number of offset-length pairs */
                      seg_ptr); /* OUT: array of offset-length pairs */
@@ -1476,15 +1495,15 @@ merge_requests(NC          *ncp,
 
         MPI_Offset gap = (*segs)[i].off + (*segs)[i].len - (*segs)[j].off;
         if (gap >= 0) { /* segments i and j overlaps */
-            if ((*segs)[i].buf_addr + (*segs)[i].len ==
-                (*segs)[j].buf_addr + gap) {
+            if (MPI_Aint_add((*segs)[i].buf_addr, (*segs)[i].len) ==
+                MPI_Aint_add((*segs)[j].buf_addr, gap)) {
                 /* buffers i and j are contiguous, merge j to i */
-                (*segs)[i].len += (*segs)[j].len - gap;
+                (*segs)[i].len = MPI_Aint_add((*segs)[i].len, (*segs)[j].len - gap);
             }
             else { /* buffers are not contiguous, reduce j's len */
                 (*segs)[i+1].off      = (*segs)[j].off + gap;
                 (*segs)[i+1].len      = (*segs)[j].len - gap;
-                (*segs)[i+1].buf_addr = (*segs)[j].buf_addr + gap;
+                (*segs)[i+1].buf_addr = MPI_Aint_add((*segs)[j].buf_addr, gap);
                 i++;
             }
         }
@@ -1751,7 +1770,7 @@ req_aggregation(NC     *ncp,
      * fileview must contain only monotonic non-decreasing file offsets. Thus
      * if the nonblocking requests interleave with each other (although not
      * overlap), then we cannot simply concatenate the filetypes of individual
-     * requests. This approach flattens the requests of "interleaved" groups
+     * requests. Codes below flatten the requests of "interleaved" groups
      * into offset-length pairs, sorts, and merges them into an aggregated
      * filetype. Similar for building an aggregated I/O buffer type.
      */
@@ -1905,28 +1924,32 @@ req_aggregation(NC     *ncp,
         else { /* this group is interleaved */
             /* flatten the interleaved requests in this group, so interleaved
              * requests can be sorted and merged into a monotonically
-             * non-decreasing filetype. For example, multiple nonblocking
-             * requests each accessing a single column of a 2D array, that each
-             * produces a filetype interleaving with others'.
+             * non-decreasing order for constructing the filetype. For example,
+             * multiple nonblocking requests each writing/reading a single
+             * column of a 2D array will produces an interleaving filetype
+             * among all processes.
              *
              * The pitfall of this flattening is the additional memory
              * requirement, as it will have to break down each request into a
-             * list of offset-length pairs, and merge all lists into a sorted
-             * list based on their offsets into an increasing order.
+             * list of offset-length pairs, and then merge all lists into a
+             * sorted list based on their offsets into an increasing order.
              *
              * Be warned! The additional memory requirement for this merging can
-             * be more than the I/O data itself. For example, each nonblocking
-             * request access a single column of a 2D array of 4-byte integer
-             * type. Each off-len pair represents only a 4-byte integer, but the
-             * off-len pair itself takes 24 bytes. Additional memory is also
-             * required for MPI arguments of displacements and blocklengths.
+             * be more than the I/O data itself. For example, in the
+             * column-wise data partitioning pattern, a process makes a
+             * nonblocking request for accessing a single column of a 2D array
+             * of 4-byte integer type. However, each element of the column is
+             * flattened into an off-len pair and this off-len pair itself
+             * takes 24 bytes, sizeof(struct off_len). Additional memory is
+             * also required for MPI arguments of displacements and
+             * blocklengths when constructing the filetype.
              */
             MPI_Offset  nsegs=0;   /* number of merged offset-length pairs */
             off_len    *segs=NULL; /* array of the offset-length pairs */
             void       *merged_buf;
 
-            /* merge all requests into sorted offset-length pairs. Note
-             * g_reqs[].offset_start and offset_end are relative to the
+            /* flatten and merge all requests into sorted offset-length pairs.
+             * Note g_reqs[].offset_start and offset_end are relative to the
              * beginning of file */
             err = merge_requests(ncp, lead_list, g_num_reqs, g_reqs,
                                  &merged_buf, &nsegs, &segs);
@@ -1957,7 +1980,7 @@ req_aggregation(NC     *ncp,
         if (i > 0) {
             /* get the buffer address of the first request in this group */
             MPI_Get_address(g_reqs[0].xbuf, &b_addr);
-            b_disps[i] = b_addr - b_begin; /* to 1st buffer of 1st group*/
+            b_disps[i] = MPI_Aint_diff(b_addr, b_begin); /* to 1st buffer of 1st group*/
         }
         b_blocklens[i] = 1;
     }
@@ -2037,10 +2060,9 @@ req_aggregation(NC     *ncp,
     }
     NCI_Free(reqs);
 
-    if (coll_indep == NC_REQ_COLL)
+    fh = ncp->independent_fh;
+    if (ncp->nprocs > 1 && coll_indep == NC_REQ_COLL)
         fh = ncp->collective_fh;
-    else
-        fh = ncp->independent_fh;
 
     /* set the MPI-IO fileview, this is a collective call */
     offset = 0;
@@ -2346,11 +2368,12 @@ mgetput(NC     *ncp,
         last_contig_req = 0; /* index of the last contiguous request */
         buf = NULL;
         /* process only valid requests */
-        for (i=0, j=0; i<num_reqs; i++) {
+        for (j=0, i=0; i<num_reqs; i++) {
             MPI_Offset req_size;
             NC_lead_req *lead = lead_list + reqs[i].lead_off;
 
-            if (fIsSet(lead->flag, NC_REQ_SKIP)) continue;
+            if (fIsSet(lead->flag, NC_REQ_SKIP))
+                continue; /* skip invalid request */
 
             req_size = reqs[i].nelems * lead->varp->xsz;
 
@@ -2371,13 +2394,16 @@ mgetput(NC     *ncp,
             if (j == 0) { /* first valid request */
                 a_last_contig = a0 = ai;
                 buf = reqs[i].xbuf;
+                disps[0] = 0;
+                j = 1;
+                continue;
             }
-            disps[j] = ai - a0;
+            disps[j] = MPI_Aint_diff(ai, a0);
 
             req_size = blocklens[last_contig_req];
             req_size += blocklens[j];
 #ifdef HAVE_MPI_LARGE_COUNT
-            if (ai - a_last_contig == blocklens[last_contig_req]) {
+            if (MPI_Aint_diff(ai, a_last_contig) == blocklens[last_contig_req]) {
                 /* user buffer of request j is contiguous from j-1
                  * we coalesce j to j-1 */
                 blocklens[last_contig_req] += blocklens[j];
@@ -2385,7 +2411,7 @@ mgetput(NC     *ncp,
 #else
             /* if req_size overflows 4-byte int, then skip coalescing */
             if (req_size <= NC_MAX_INT &&
-                ai - a_last_contig == blocklens[last_contig_req]) {
+                MPI_Aint_diff(ai, a_last_contig) == blocklens[last_contig_req]) {
                 /* user buffer of request j is contiguous from j-1
                  * we coalesce j to j-1 */
                 blocklens[last_contig_req] += blocklens[j];
@@ -2395,7 +2421,7 @@ mgetput(NC     *ncp,
                 /* not contiguous from request last_contig_req */
                 last_contig_req++;
                 a_last_contig = ai;
-                disps[last_contig_req] = ai - a0;
+                disps[last_contig_req] = MPI_Aint_diff(ai, a0);
                 blocklens[last_contig_req] = blocklens[i];
             }
             j++;
@@ -2459,10 +2485,9 @@ mgetput(NC     *ncp,
 mpi_io:
     NCI_Free(reqs);
 
-    if (coll_indep == NC_REQ_COLL)
+    fh = ncp->independent_fh;
+    if (ncp->nprocs > 1 && coll_indep == NC_REQ_COLL)
         fh = ncp->collective_fh;
-    else
-        fh = ncp->independent_fh;
 
     /* set the MPI-IO fileview, this is a collective call */
     err = ncmpio_file_set_view(ncp, fh, &offset, filetype);
