@@ -35,7 +35,7 @@
 }
 
 #define CHECK_VAR_DIFF(itype) {                                          \
-    int indx, esize, nelems;                                             \
+    size_t indx, esize, nelems;                                          \
     itype *b1 = (itype*)buf[0];                                          \
     itype *b2 = (itype*)buf[1];                                          \
     esize = sizeof(itype);                                               \
@@ -64,6 +64,34 @@
     chunk_off += nelems;                                                 \
 }
 
+#define CHECK_VAR_DIFF_UNSIGNED(itype) {                                 \
+    size_t indx, esize, nelems;                                          \
+    itype *b1 = (itype*)buf[0];                                          \
+    itype *b2 = (itype*)buf[1];                                          \
+    esize = sizeof(itype);                                               \
+    nelems = rdLen[0] / esize;                                           \
+    if (esize > 1 && is_little_endian) {                                 \
+        swapn(b1, nelems, esize);                                        \
+        swapn(b2, nelems, esize);                                        \
+    }                                                                    \
+    for (indx=0; indx<nelems; indx++) {                                  \
+        double abs_max, diff, ratio;                                     \
+        if ( b1[indx] == b2[indx] ) continue;                            \
+        abs_max = (b1[indx] > b2[indx]) ? b1[indx] : b2[indx];           \
+        diff = b1[indx] - b2[indx];                                      \
+        diff = (diff >= 0) ? diff : -diff;                               \
+        ratio = diff /  abs_max;                                         \
+        if (diff <= tolerance_difference || ratio <= tolerance_ratio)    \
+            continue;                                                    \
+        /* fail to meet both tolerance errors */                         \
+        worst = chunk_off + indx; /* mark a difference is found */       \
+        worst1 = b1[indx];                                               \
+        worst2 = b2[indx];                                               \
+        break;                                                           \
+    }                                                                    \
+    chunk_off += nelems;                                                 \
+}
+
 #define PRINTF_VAR_DIFF(itype) {                                         \
     int esize = sizeof(itype);                                           \
     itype *b1 = (itype*)str[0];                                          \
@@ -85,7 +113,7 @@
 static void
 swapn(void   *buf,
       size_t  nelems,  /* number of elements in buf[] */
-      int     esize)   /* byte size of each element */
+      size_t  esize)   /* byte size of each element */
 {
     size_t i;
 
@@ -145,7 +173,7 @@ usage(char *progname)
                    and ratio is relative element-wise difference defined\n\
                    as |x - y|/max(|x|, |y|)\n\
   file1 file2      File names of two input netCDF files to be compared\n\
-  *PnetCDF library version 1.12.3 of February 21, 2022\n"
+  *PnetCDF library version PNETCDF_RELEASE_VERSION of PNETCDF_RELEASE_DATE\n"
 
     printf("  %s [-b] [-q] [-h] [-v ...] [-t diff,ratio] file1 file2\n%s",
            progname, USAGE);
@@ -213,7 +241,7 @@ int main(int argc, char **argv)
     extern int optind;
     char *str, *ptr;
     size_t nbytes;
-    int i, j, k, m, n, c, err, verbose, quiet, isDiff, nChunks;
+    int i, j, k, m, n, c, err, verbose, quiet, isDiff;
     int fd[2], nvars[2], ndims[2], nattrs[2], check_tolerance;
     int cmp_nvars, check_header, check_variable_list, check_entire_file;
     long long numVarDIFF=0, numHeadDIFF=0, numDIFF;
@@ -804,7 +832,7 @@ cmp_vars:
     for (i=0; i<cmp_nvars; i++) {
         char *var_name;
         int varid[2], *dimids[2], isRecVar[2];
-        long long remainLen, varsize[2], offset[2];
+        long long remainLen, nChunks, varsize[2], offset[2];
         off_t seek_ret;
         nc_type xtype[2];
         NC_var  *var[2];
@@ -971,15 +999,15 @@ cmp_vars:
                 if (check_tolerance && worst == -1) {
                          if (xtype[0] == NC_CHAR)   CHECK_VAR_DIFF(char)
                     else if (xtype[0] == NC_BYTE)   CHECK_VAR_DIFF(signed char)
-                    else if (xtype[0] == NC_UBYTE)  CHECK_VAR_DIFF(unsigned char)
+                    else if (xtype[0] == NC_UBYTE)  CHECK_VAR_DIFF_UNSIGNED(unsigned char)
                     else if (xtype[0] == NC_SHORT)  CHECK_VAR_DIFF(short)
-                    else if (xtype[0] == NC_USHORT) CHECK_VAR_DIFF(unsigned short)
+                    else if (xtype[0] == NC_USHORT) CHECK_VAR_DIFF_UNSIGNED(unsigned short)
                     else if (xtype[0] == NC_INT)    CHECK_VAR_DIFF(int)
-                    else if (xtype[0] == NC_UINT)   CHECK_VAR_DIFF(unsigned int)
+                    else if (xtype[0] == NC_UINT)   CHECK_VAR_DIFF_UNSIGNED(unsigned int)
                     else if (xtype[0] == NC_FLOAT)  CHECK_VAR_DIFF(float)
                     else if (xtype[0] == NC_DOUBLE) CHECK_VAR_DIFF(double)
                     else if (xtype[0] == NC_INT64)  CHECK_VAR_DIFF(long long)
-                    else if (xtype[0] == NC_UINT64) CHECK_VAR_DIFF(unsigned long long)
+                    else if (xtype[0] == NC_UINT64) CHECK_VAR_DIFF_UNSIGNED(unsigned long long)
                     if (worst != -1) {
                         r = numrecs; /* break both loops k and r */
                         break;
