@@ -12,6 +12,9 @@ NCMPIDIFF=../../src/utils/ncmpidiff/ncmpidiff
 
 outfile=`basename $1`
 
+# remove file system type prefix if there is any
+OUTDIR=`echo "$TESTOUTDIR" | cut -d: -f2-`
+
 # echo "PNETCDF_DEBUG = ${PNETCDF_DEBUG}"
 if test ${PNETCDF_DEBUG} = 1 ; then
    safe_modes="0 1"
@@ -19,6 +22,9 @@ else
    safe_modes="0"
 fi
 
+# prevent user environment setting of PNETCDF_HINTS to interfere
+unset PNETCDF_HINTS
+
 for j in ${safe_modes} ; do
     export PNETCDF_SAFE_MODE=$j
     # echo "---- set PNETCDF_SAFE_MODE ${PNETCDF_SAFE_MODE}"
@@ -27,7 +33,9 @@ for j in ${safe_modes} ; do
     # echo ""
 
     if test "x${ENABLE_BURST_BUFFER}" = x1 ; then
-       # echo "---- testing burst buffering"
+       echo ""
+       echo "---- testing burst buffering"
+
        export PNETCDF_HINTS="nc_burst_buf=enable;nc_burst_buf_dirname=${TESTOUTDIR};nc_burst_buf_overwrite=enable"
        ${TESTSEQRUN} $1              ${TESTOUTDIR}/$outfile.bb.nc
        unset PNETCDF_HINTS
@@ -41,4 +49,6 @@ for j in ${safe_modes} ; do
        fi
     fi
 done
+rm -f ${OUTDIR}/$outfile.nc
+rm -f ${OUTDIR}/$outfile.bb.nc
 
