@@ -7,6 +7,9 @@
 # Exit immediately if a command exits with a non-zero status.
 set -e
 
+# prevent user environment setting of PNETCDF_HINTS to interfere
+unset PNETCDF_HINTS
+
 ${TESTSEQRUN} ./test_inq_format ${srcdir}
 
 # the followings check files with corrupted header
@@ -16,7 +19,8 @@ ${TESTSEQRUN} ./tst_corrupt ${srcdir}
 # echo ""
 
 if test "x${ENABLE_BURST_BUFFER}" = x1 ; then
-    # echo "---- testing burst buffering"
+    echo ""
+    echo "---- testing burst buffering"
 
     export PNETCDF_HINTS="nc_burst_buf=enable;nc_burst_buf_dirname=${TESTOUTDIR};nc_burst_buf_overwrite=enable"
 
