@@ -7,7 +7,7 @@
 
 #include <stdio.h>
 #include <limits.h>
-#include <string.h>
+#include <string.h> /* strchr(), strerror(), strdup(), strcpy(), strlen() */
 #include <mpi.h>
 
 #include <pnetcdf.h>
@@ -255,3 +255,33 @@ inq_env_hint(char *hint_key, char **hint_value)
     return 0;
 }
 
+/* File system types recognized by ROMIO in MPICH 4.0.0 */
+static const char* fstypes[] = {"ufs", "nfs", "xfs", "pvfs2", "gpfs", "panfs", "lustre", "daos", "testfs", "ime", "quobyte", NULL};
+
+/* Return a pointer to filename by removing the file system type prefix name if
+ * there is any.  For example, when filename = "lustre:/home/foo/testfile.nc",
+ * remove "lustre:" to return a pointer to "/home/foo/testfile.nc", so the name
+ * can be used in POSIX open() calls.
+ */
+char* remove_file_system_type_prefix(const char *filename)
+{
+    char *ret_filename = (char*)filename;
+
+    if (filename == NULL) return NULL;
+
+    if (strchr(filename, ':') != NULL) { /* there is a prefix end with ':' */
+        /* check if prefix is one of recognized file system types */
+        int i=0;
+        while (fstypes[i] != NULL) {
+            size_t prefix_len = strlen(fstypes[i]);
+            if (!strncmp(filename, fstypes[i], prefix_len)) { /* found */
+                ret_filename += prefix_len + 1;
+                break;
+            }
+            i++;
+        }
+    }
+
+    return ret_filename;
+}
+
