@@ -30,8 +30,11 @@
 int main(int argc, char** argv)
 {
     char filename[256], str[32];
-    int i, rank, nprocs, err, nerrs=0, verbose=0;
+    int i, rank, nprocs, err, nerrs=0;
     int ncid, cmode, *varid, *dimids, intBuf[1];
+#ifdef PNC_MALLOC_TRACE
+    int verbose=0;
+#endif
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -131,6 +134,7 @@ int main(int argc, char** argv)
     free(varid);
     free(dimids);
 
+#ifdef PNC_MALLOC_TRACE
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
     err = ncmpi_inq_malloc_size(&malloc_size);
@@ -148,6 +152,7 @@ int main(int argc, char** argv)
         printf("\n%d: PnetCDF internal memory footprint high water mark %.2f MB\n",
                rank, (float)malloc_size/1048576);
     }
+#endif
 
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
     if (rank == 0) {
