@@ -38,7 +38,8 @@ static int verbose;
 }
 
 #define CHECK_GLOBAL_ATT_DIFF(type, func, nctype) {                          \
-    int   pos, len = attlen1 * sizeof(type);                                 \
+    int pos;                                                                 \
+    size_t len = sizeof(type) * attlen1;                                     \
     type *b1 = (type *)malloc(len);                                          \
     type *b2 = (type *)malloc(len);                                          \
     err = func(ncid1, NC_GLOBAL, name1, b1);                                 \
@@ -56,7 +57,8 @@ static int verbose;
 }
 
 #define CHECK_VAR_ATT_DIFF(type, func, nctype) {                             \
-    int   pos, len = attlen1 * sizeof(type);                                 \
+    int pos;                                                                 \
+    size_t len = sizeof(type) * attlen1;                                     \
     type *b1 = (type *)malloc(len);                                          \
     type *b2 = (type *)malloc(len);                                          \
     err = func(ncid1, i, name1, b1);                                         \
@@ -75,7 +77,8 @@ static int verbose;
 
 
 #define CHECK_VAR_DIFF(type, func, nctype) {                                 \
-    int   pos, len = varsize * sizeof(type);                                 \
+    int pos;                                                                 \
+    size_t len = sizeof(type) * varsize;                                     \
     type *b1 = (type *)malloc(len);                                          \
     type *b2 = (type *)malloc(len);                                          \
     err = func(ncid1, i, start, shape, b1);                                  \
@@ -512,6 +515,12 @@ int main(int argc, char **argv)
         err = ncmpi_enddef(ncid);
         CHECK_ERR
 
+#ifdef STRONGER_CONSISTENCY
+        ncmpi_sync(ncid);
+        MPI_Barrier(MPI_COMM_WORLD);
+        ncmpi_sync(ncid);
+#endif
+
         if (k == 0) {
             if (rank == 0 && verbose)
                 printf("*** Testing to write 2 non-record variables and 2 record variables by using ncmpi_put_vara_all() ...");
@@ -522,6 +531,12 @@ int main(int argc, char **argv)
                     CHECK_ERR
                 }
             }
+#ifdef STRONGER_CONSISTENCY
+            ncmpi_sync(ncid);
+            MPI_Barrier(MPI_COMM_WORLD);
+            ncmpi_sync(ncid);
+#endif
+
             for (i=0; i<nvars; i++){
                 err = ncmpi_put_vara_all(ncid, varid[i], starts[i], counts[i], buf[i], bufcounts[i], MPI_INT);
                 CHECK_ERR
