@@ -42,7 +42,7 @@
 
           character(LEN=256) filename, cmd, msg
           integer err, ierr, nprocs, rank, cmode, ncid, nerrs, get_args
-          integer varid(7), dimid(3), dimid_1D(1), dimid_2D(2)
+          integer varid(7), dimid(3), dimid_1D(1), dimid_2D(2), old_mode
           integer(kind=MPI_OFFSET_KIND) expected_recsize, recsize
           integer(kind=MPI_OFFSET_KIND) malloc_size, sum_size
 
@@ -106,6 +106,10 @@
           err = nf90mpi_def_var(ncid, "FIX_VAR_3", NF90_INT, dimid_1D, varid(7))
           call check(err, 'In nf90mpi_def_var: FIX_VAR_3')
 
+          ! set fill mode, so ncmpidiff can compare 2 output files without error
+          err = nf90mpi_set_fill(ncid, NF_FILL, old_mode)
+          call check(err, 'In nf90mpi_set_fill: ')
+
           ! do not forget to exit define mode
           err = nf90mpi_enddef(ncid)
           call check(err, 'In nf90mpi_enddef: ')
