@@ -8,49 +8,96 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h> /* strcasecmp() */
 #include <libgen.h> /* basename() */
 #include <pnetcdf.h>
 
 #include <testutils.h>
 
-/*----< main() >------------------------------------------------------------*/
-int main(int argc, char **argv)
+static int
+check_read_contents(float *rh)
+{
+    int i, j;
+    float k;
+
+    /* check the contents of read */
+    k = 0.0;
+    for (i=0; i<6; i++) {
+        for (j=0; j<4; j++) {
+            if (rh[j*6+i] != k) {
+#ifdef PRINT_ERR_ON_SCREEN
+                printf("Error at %s:%d : expect rh[%d][%d]=%f but got %f\n",
+                __FILE__,__LINE__,j,i,k,rh[j*6+i]);
+#endif
+                return 1;
+            }
+            k += 1.0;
+        }
+    }
+#ifdef PRINT_ON_SCREEN
+    /* print the contents of read */
+    for (j=0; j<4; j++) {
+        printf("[%2d]: ",j);
+        for (i=0; i<6; i++) {
+            printf("%5.1f",rh[j][i]);
+        }
+        printf("\n");
+    }
+#endif
+    /* the stdout should be:
+           [ 0]:   0.0  4.0  8.0 12.0 16.0 20.0
+           [ 1]:   1.0  5.0  9.0 13.0 17.0 21.0
+           [ 2]:   2.0  6.0 10.0 14.0 18.0 22.0
+           [ 3]:   3.0  7.0 11.0 15.0 19.0 23.0
+     */
+    return 0;
+}
+
+static int
+check_write_contents(signed char *varT)
+{
+    int i, j;
+    /* the output from command "ncmpidump -v var test.nc" should be:
+           var =
+            50, 56, 62, 68,
+            51, 57, 63, 69,
+            52, 58, 64, 70,
+            53, 59, 65, 71,
+            54, 60, 66, 72,
+            55, 61, 67, 73 ;
+     */
+
+    /* check if the contents of write buffer have been altered */
+    for (j=0; j<4; j++) {
+        for (i=0; i<6; i++) {
+            if (varT[j*6+i] != j*6+i + 50) {
+#ifdef PRINT_ERR_ON_SCREEN
+                /* this error is a pnetcdf internal error, if occurs */
+                printf("Error at line %d in %s: expecting varT[%d][%d]=%d but got %d\n",
+                __LINE__,__FILE__,j,i,j*6+i + 50,varT[j*6+i]);
+#endif
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
+
+static int
+tst_fmt(char *filename, int cmode)
 {
     int i, j, err, nerrs=0, rank, nprocs;
     int ncid, dimid[2], varid, req, status;
-
     MPI_Offset start[2], count[2], stride[2], imap[2];
-    int   var[6][4];
-    float k, rh[4][6];
-    signed char  varT[4][6];
-    char filename[256];
+    int          var[6][4];
+    float         rh[4][6];
+    signed char varT[4][6];
 
-    MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
 
-    if (argc > 2) {
-        if (!rank) printf("Usage: %s [filename]\n",argv[0]);
-        MPI_Finalize();
-        return 1;
-    }
-    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
-    else           strcpy(filename, "testfile.nc");
-
-    if (rank == 0) {
-        char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
-        sprintf(cmd_str, "*** TESTING C   %s for get/put varm ", basename(argv[0]));
-        printf("%-66s ------ ", cmd_str); fflush(stdout);
-        free(cmd_str);
-    }
-
-#ifdef DEBUG
-    if (nprocs > 1 && rank == 0)
-        printf("Warning: %s is designed to run on 1 process\n", argv[0]);
-#endif
-
-    err = ncmpi_create(MPI_COMM_WORLD, filename, NC_CLOBBER | NC_64BIT_DATA,
-                       MPI_INFO_NULL, &ncid); CHECK_ERR
+    cmode |= NC_CLOBBER;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid); CHECK_ERR
 
     /* define a variable of a 6 x 4 integer array in the nc file */
     err = ncmpi_def_dim(ncid, "Y", 6, &dimid[0]); CHECK_ERR
@@ -86,81 +133,47 @@ int main(int argc, char **argv)
     stride[0] = 1; stride[1] = 1;
       imap[0] = 1;   imap[1] = 6;   /* would be {4, 1} if not transposing */
 
-    for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+    if (cmode & NC_NETCDF4) {
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_get_varm_float_all(ncid, varid, start, count, stride, imap,
+                                       &rh[0][0]); CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
 
-    err = ncmpi_iget_varm_float(ncid, varid, start, count, stride, imap, &rh[0][0], &req); CHECK_ERR
-
-    err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
-    err = status; CHECK_ERR
-
-    /* check the contents of read */
-    k = 0.0;
-    for (i=0; i<6; i++) {
-        for (j=0; j<4; j++) {
-            if (rh[j][i] != k) {
-#ifdef PRINT_ERR_ON_SCREEN
-                printf("Error at line %d in %s: expecting rh[%d][%d]=%f but got %f\n",
-                __LINE__,__FILE__,j,i,k,rh[j][i]);
-#endif
-                nerrs++;
-                break;
-            }
-            k += 1.0;
-        }
+        /* test when stride == NULL and imap != NULL */
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_get_varm_float_all(ncid, varid, start, count, NULL, imap,
+                                       &rh[0][0]); CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
     }
-#ifdef PRINT_ON_SCREEN
-    /* print the contents of read */
-    for (j=0; j<4; j++) {
-        printf("[%2d]: ",j);
-        for (i=0; i<6; i++) {
-            printf("%5.1f",rh[j][i]);
-        }
-        printf("\n");
+    else {
+        /* test nonblocking API */
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_iget_varm_float(ncid, varid, start, count, stride, imap,
+                                    &rh[0][0], &req); CHECK_ERR
+        err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
+        err = status; CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
+
+        /* test when stride == NULL and imap != NULL */
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_iget_varm_float(ncid, varid, start, count, NULL, imap,
+                                    &rh[0][0], &req); CHECK_ERR
+        err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
+        err = status; CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
+
+        /* test blocking API */
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_get_varm_float_all(ncid, varid, start, count, stride, imap,
+                                       &rh[0][0]); CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
+
+        /* test when stride == NULL and imap != NULL */
+        for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
+        err = ncmpi_get_varm_float_all(ncid, varid, start, count, NULL, imap,
+                                       &rh[0][0]); CHECK_ERR
+        nerrs += check_read_contents(&rh[0][0]);
     }
-#endif
-    /* the stdout should be:
-           [ 0]:   0.0  4.0  8.0 12.0 16.0 20.0
-           [ 1]:   1.0  5.0  9.0 13.0 17.0 21.0
-           [ 2]:   2.0  6.0 10.0 14.0 18.0 22.0
-           [ 3]:   3.0  7.0 11.0 15.0 19.0 23.0
-     */
-
-    for (i=0; i<6; i++) for (j=0; j<4; j++) rh[j][i] = -1.0;
-
-    err = ncmpi_get_varm_float_all(ncid, varid, start, count, stride, imap, &rh[0][0]); CHECK_ERR
-
-    /* check the contents of read */
-    k = 0.0;
-    for (i=0; i<6; i++) {
-        for (j=0; j<4; j++) {
-            if (rh[j][i] != k) {
-#ifdef PRINT_ERR_ON_SCREEN
-                printf("Error at line %d in %s: expecting rh[%d][%d]=%f but got %f\n",
-                __LINE__,__FILE__,j,i,k,rh[j][i]);
-#endif
-                nerrs++;
-                break;
-            }
-            k += 1.0;
-        }
-    }
-#ifdef PRINT_ON_SCREEN
-    /* print the contents of read */
-    for (j=0; j<4; j++) {
-        printf("[%2d]: ",j);
-        for (i=0; i<6; i++) {
-            printf("%5.1f",rh[j][i]);
-        }
-        printf("\n");
-    }
-#endif
-    /* the stdout should be:
-           [ 0]:   0.0  4.0  8.0 12.0 16.0 20.0
-           [ 1]:   1.0  5.0  9.0 13.0 17.0 21.0
-           [ 2]:   2.0  6.0 10.0 14.0 18.0 22.0
-           [ 3]:   3.0  7.0 11.0 15.0 19.0 23.0
-     */
-
 
     err = ncmpi_close(ncid); CHECK_ERR
 
@@ -190,54 +203,88 @@ int main(int argc, char **argv)
     imap[0]   = 1; imap[1]   = 6;   /* would be {4, 1} if not transposing */
     if (rank > 0) count[0] = count[1] = 0;
 
-    err = ncmpi_iput_varm_schar(ncid, varid, start, count, stride, imap, &varT[0][0], &req); CHECK_ERR
-
-    err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
-    err = status; CHECK_ERR
-
-    /* the output from command "ncmpidump -v var test.nc" should be:
-           var =
-            50, 56, 62, 68,
-            51, 57, 63, 69,
-            52, 58, 64, 70,
-            53, 59, 65, 71,
-            54, 60, 66, 72,
-            55, 61, 67, 73 ;
-     */
-
-    /* check if the contents of write buffer have been altered */
-    for (j=0; j<4; j++) {
-        for (i=0; i<6; i++) {
-            if (varT[j][i] != j*6+i + 50) {
-#ifdef PRINT_ERR_ON_SCREEN
-                /* this error is a pnetcdf internal error, if occurs */
-                printf("Error at line %d in %s: expecting varT[%d][%d]=%d but got %d\n",
-                __LINE__,__FILE__,j,i,j*6+i + 50,varT[j][i]);
-#endif
-                nerrs++;
-                break;
-            }
-        }
+    if (cmode & NC_NETCDF4) {
+        err = ncmpi_put_varm_schar_all(ncid, varid, start, count, stride,
+                                       imap, &varT[0][0]); CHECK_ERR
+        nerrs += check_write_contents(&varT[0][0]);
     }
-    err = ncmpi_put_varm_schar_all(ncid, varid, start, count, stride, imap, &varT[0][0]); CHECK_ERR
+    else {
+        /* test nonblocking API */
+        err = ncmpi_iput_varm_schar(ncid, varid, start, count, stride,
+                                    imap, &varT[0][0], &req); CHECK_ERR
+        err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
+        err = status; CHECK_ERR
+        nerrs += check_write_contents(&varT[0][0]);
 
-    /* check if the contents of write buffer have been altered */
-    for (j=0; j<4; j++) {
-        for (i=0; i<6; i++) {
-            if (varT[j][i] != j*6+i + 50) {
-#ifdef PRINT_ERR_ON_SCREEN
-                /* this error is a pnetcdf internal error, if occurs */
-                printf("Error at line %d in %s: expecting varT[%d][%d]=%d but got %d\n",
-                __LINE__,__FILE__,j,i,j*6+i + 50,varT[j][i]);
-#endif
-                nerrs++;
-                break;
-            }
-        }
+        /* test when stride == NULL and imap != NULL */
+        err = ncmpi_iput_varm_schar(ncid, varid, start, count, NULL,
+                                    imap, &varT[0][0], &req); CHECK_ERR
+        err = ncmpi_wait_all(ncid, 1, &req, &status); CHECK_ERR
+        err = status; CHECK_ERR
+        nerrs += check_write_contents(&varT[0][0]);
+
+        /* test blocking API */
+        err = ncmpi_put_varm_schar_all(ncid, varid, start, count, stride, imap,
+                                       &varT[0][0]); CHECK_ERR
+        nerrs += check_write_contents(&varT[0][0]);
+
+        /* test when stride == NULL and imap != NULL */
+        err = ncmpi_put_varm_schar_all(ncid, varid, start, count, NULL, imap,
+                                       &varT[0][0]); CHECK_ERR
+        nerrs += check_write_contents(&varT[0][0]);
     }
 
     err = ncmpi_close(ncid); CHECK_ERR
 
+    return nerrs;
+}
+
+/*----< main() >------------------------------------------------------------*/
+int main(int argc, char **argv)
+{
+    char filename[256], *hint_value;
+    int err, nerrs=0, rank, nprocs, bb_enabled=0;
+
+    MPI_Init(&argc, &argv);
+    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
+
+    if (argc > 2) {
+        if (!rank) printf("Usage: %s [filename]\n",argv[0]);
+        MPI_Finalize();
+        return 1;
+    }
+    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
+    else           strcpy(filename, "testfile.nc");
+
+    if (rank == 0) {
+        char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
+        sprintf(cmd_str, "*** TESTING C   %s for get/put varm ", basename(argv[0]));
+        printf("%-66s ------ ", cmd_str); fflush(stdout);
+        free(cmd_str);
+    }
+
+#ifdef DEBUG
+    if (nprocs > 1 && rank == 0)
+        printf("Warning: %s is designed to run on 1 process\n", argv[0]);
+#endif
+
+    /* check whether burst buffering is enabled */
+    if (inq_env_hint("nc_burst_buf", &hint_value)) {
+        if (strcasecmp(hint_value, "enable") == 0) bb_enabled = 1;
+        free(hint_value);
+    }
+
+    nerrs += tst_fmt(filename, 0);
+    nerrs += tst_fmt(filename, NC_64BIT_OFFSET);
+    if (!bb_enabled) {
+#ifdef ENABLE_NETCDF4
+        nerrs += tst_fmt(filename, NC_NETCDF4);
+        nerrs += tst_fmt(filename, NC_NETCDF4 | NC_CLASSIC_MODEL);
+#endif
+    }
+    nerrs += tst_fmt(filename, NC_64BIT_DATA);
+
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
     err = ncmpi_inq_malloc_size(&malloc_size);
